@@ -1,5 +1,5 @@
 /*
- * $Id: desktop.c,v 1.7.2.2 2002/01/02 17:27:50 srittau Exp $
+ * $Id: desktop.c,v 1.18 2002/10/05 14:04:47 didg Exp $
  *
  * See COPYRIGHT.
  */
@@ -8,7 +8,7 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#include <sys/syslog.h>
+#include <atalk/logger.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/time.h>
@@ -40,6 +40,9 @@
 #include "fork.h"
 #include "globals.h"
 #include "desktop.h"
+#ifdef FILE_MANGLING
+#include "mangle.h"
+#endif /* CNID_DB */
 
 int afp_opendt(obj, ibuf, ibuflen, rbuf, rbuflen )
 AFPObj      *obj;
@@ -106,7 +109,7 @@ u_char	creator[ 4 ];
             *adts = '/';
 
             if (( si.sdt_fd = open( dtf, flags, ad_mode( dtf, mode ))) < 0 ) {
-                syslog( LOG_ERR, "iconopen: open %s: %s", dtf, strerror(errno) );
+                LOG(log_error, logtype_afpd, "iconopen: open %s: %s", dtf, strerror(errno) );
                 return -1;
             }
         } else {
@@ -171,7 +174,7 @@ int		ibuflen, *rbuflen;
     if (lseek( si.sdt_fd, (off_t) 0L, SEEK_SET ) < 0) {
         close(si.sdt_fd);
         si.sdt_fd = -1;
-        syslog( LOG_ERR, "afp_addicon: lseek: %s", strerror(errno) );
+        LOG(log_error, logtype_afpd, "afp_addicon: lseek: %s", strerror(errno) );
         cc = AFPERR_PARAM;
         goto addicon_err;
     }
@@ -206,7 +209,7 @@ int		ibuflen, *rbuflen;
         }
 
         if ( lseek( si.sdt_fd, (off_t) rsize, SEEK_CUR ) < 0 ) {
-            syslog( LOG_ERR, "afp_addicon: lseek: %s", strerror(errno) );
+            LOG(log_error, logtype_afpd, "afp_addicon: lseek: %s", strerror(errno) );
             cc = AFPERR_PARAM;
         }
     }
@@ -216,7 +219,7 @@ int		ibuflen, *rbuflen;
      */
 addicon_err:
     if ( cc < 0 ) {
-        syslog( LOG_ERR, "afp_addicon: %s", strerror(errno) );
+        LOG(log_error, logtype_afpd, "afp_addicon: %s", strerror(errno) );
         if (obj->proto == AFPPROTO_DSI) {
             dsi_writeinit(obj->handle, rbuf, buflen);
             dsi_writeflush(obj->handle);
@@ -257,7 +260,7 @@ addicon_err:
         }
 
         if ( writev( si.sdt_fd, iov, iovcnt ) < 0 ) {
-            syslog( LOG_ERR, "afp_addicon: writev: %s", strerror(errno) );
+            LOG(log_error, logtype_afpd, "afp_addicon: writev: %s", strerror(errno) );
             return( AFPERR_PARAM );
         }
         break;
@@ -270,13 +273,13 @@ addicon_err:
 
             /* add headers at end of file */
             if ((cc == 0) && (write(si.sdt_fd, imh, sizeof(imh)) < 0)) {
-                syslog(LOG_ERR, "afp_addicon: write: %s", strerror(errno));
+                LOG(log_error, logtype_afpd, "afp_addicon: write: %s", strerror(errno));
                 dsi_writeflush(dsi);
                 return AFPERR_PARAM;
             }
 
             if ((cc = write(si.sdt_fd, rbuf, iovcnt)) < 0) {
-                syslog(LOG_ERR, "afp_addicon: write: %s", strerror(errno));
+                LOG(log_error, logtype_afpd, "afp_addicon: write: %s", strerror(errno));
                 dsi_writeflush(dsi);
                 return AFPERR_PARAM;
             }
@@ -288,7 +291,7 @@ addicon_err:
                 }
 
                 if ((cc = write(si.sdt_fd, rbuf, iovcnt)) < 0) {
-                    syslog(LOG_ERR, "afp_addicon: write: %s", strerror(errno));
+                    LOG(log_error, logtype_afpd, "afp_addicon: write: %s", strerror(errno));
                     dsi_writeflush(dsi);
                     return AFPERR_PARAM;
                 }
@@ -402,7 +405,7 @@ int		ibuflen, *rbuflen;
         memcpy( &bsize, ih + 10, sizeof( bsize ));
         bsize = ntohs(bsize);
         if ( lseek( si.sdt_fd, (off_t) bsize, SEEK_CUR ) < 0 ) {
-            syslog( LOG_ERR, "afp_iconinfo: lseek: %s", strerror(errno) );
+            LOG(log_error, logtype_afpd, "afp_iconinfo: lseek: %s", strerror(errno) );
             return( AFPERR_PARAM );
         }
         if ( si.sdt_index == iindex ) {
@@ -461,7 +464,7 @@ int		ibuflen, *rbuflen;
     if ( lseek( si.sdt_fd, (off_t) 0L, SEEK_SET ) < 0 ) {
         close(si.sdt_fd);
         si.sdt_fd = -1;
-        syslog(LOG_ERR, "afp_geticon: lseek: %s", strerror(errno));
+        LOG(log_error, logtype_afpd, "afp_geticon: lseek: %s", strerror(errno));
         return( AFPERR_PARAM );
     }
 
@@ -477,14 +480,14 @@ int		ibuflen, *rbuflen;
         memcpy( &rsize, ih + 10, sizeof( rsize ));
         rsize = ntohs( rsize );
         if ( lseek( si.sdt_fd, (off_t) rsize, SEEK_CUR ) < 0 ) {
-            syslog( LOG_ERR, "afp_geticon: lseek: %s", strerror(errno) );
+            LOG(log_error, logtype_afpd, "afp_geticon: lseek: %s", strerror(errno) );
             return( AFPERR_PARAM );
         }
         offset += rsize;
     }
 
     if ( rc < 0 ) {
-        syslog(LOG_ERR, "afp_geticon: read: %s", strerror(errno));
+        LOG(log_error, logtype_afpd, "afp_geticon: read: %s", strerror(errno));
         return( AFPERR_PARAM );
     }
 
@@ -551,7 +554,7 @@ geticon_done:
         return AFP_OK;
 
 geticon_exit:
-        syslog(LOG_INFO, "afp_geticon: %s", strerror(errno));
+        LOG(log_info, logtype_afpd, "afp_geticon: %s", strerror(errno));
         dsi_readdone(dsi);
         obj->exit(1);
         return AFP_OK;
@@ -611,6 +614,10 @@ char *mtoupath(const struct vol *vol, char *mpath)
         return( "." );
     }
 
+#ifdef FILE_MANGLING
+    mpath = demangle(vol, mpath);
+#endif /* FILE_MANGLING */
+
     m = mpath;
     u = upath;
     while ( *m != '\0' ) {
@@ -626,6 +633,17 @@ char *mtoupath(const struct vol *vol, char *mpath)
         if (vol->v_mtoupage && ((*m & 0x80) ||
                                 vol->v_flags & AFPVOL_MAPASCII)) {
             *u = vol->v_mtoupage->map[(unsigned char) *m].value;
+            if (!*u && *m) {
+                /* if conversion failed, encode in hex
+                 * to prevent silly truncation
+                 * H.P. Jansen <hpj@urpla.net> */
+#ifdef DEBUG
+                LOG(log_debug, logtype_afpd, "mtoupath: hex encode: 0x%x", (unsigned char) *m);
+#endif /* DEBUG */
+                *u++ = ':';
+                *u++ = hexdig[ ( *m & 0xf0 ) >> 4 ];
+                *u = hexdig[ *m & 0x0f ];
+            }
         } else
 #endif /* 1 */
 #if AD_VERSION == AD_VERSION1
@@ -651,6 +669,10 @@ char *mtoupath(const struct vol *vol, char *mpath)
     }
     *u = '\0';
 
+#ifdef DEBUG
+    LOG(log_debug, logtype_afpd, "mtoupath: '%s':'%s'", mpath, upath);
+#endif /* DEBUG */
+
     return( upath );
 }
 
@@ -694,6 +716,15 @@ char *utompath(const struct vol *vol, char *upath)
         m++;
     }
     *m = '\0';
+
+#ifdef FILE_MANGLING
+    strcpy(mpath,mangle(vol, mpath));
+#endif /* FILE_MANGLING */
+
+#ifdef DEBUG
+    LOG(log_debug, logtype_afpd, "utompath: '%s':'%s'", upath, mpath);
+#endif /* DEBUG */
+
     return( mpath );
 }
 
@@ -706,7 +737,7 @@ int		ibuflen, *rbuflen;
     struct vol		*vol;
     struct dir		*dir;
     struct ofork        *of;
-    char		*path, *name;
+    char		*path, *name, *upath;
     int			clen;
     u_int32_t           did;
     u_int16_t		vid;
@@ -722,7 +753,7 @@ int		ibuflen, *rbuflen;
 
     memcpy( &did, ibuf, sizeof( did ));
     ibuf += sizeof( did );
-    if (( dir = dirsearch( vol, did )) == NULL ) {
+    if (( dir = dirlookup( vol, did )) == NULL ) {
         return( AFPERR_NOOBJ );
     }
 
@@ -736,13 +767,13 @@ int		ibuflen, *rbuflen;
 
     clen = (u_char)*ibuf++;
     clen = min( clen, 199 );
-
-    if ((*path == '\0') || !(of = of_findname(vol, curdir, path))) {
+    upath = mtoupath( vol, path );
+    if ((*path == '\0') || !(of = of_findname(upath, NULL))) {
         memset(&ad, 0, sizeof(ad));
         adp = &ad;
     } else
         adp = of->of_ad;
-    if (ad_open( mtoupath( vol, path ), vol_noadouble(vol) |
+    if (ad_open( upath , vol_noadouble(vol) |
                  (( *path == '\0' ) ? ADFLAGS_HF|ADFLAGS_DIR : ADFLAGS_HF),
                  O_RDWR|O_CREAT, 0666, adp) < 0 ) {
         return( AFPERR_ACCESS );
@@ -775,7 +806,7 @@ int		ibuflen, *rbuflen;
     struct vol		*vol;
     struct dir		*dir;
     struct ofork        *of;
-    char		*path;
+    char		*path, *upath;
     u_int32_t		did;
     u_int16_t		vid;
 
@@ -790,7 +821,7 @@ int		ibuflen, *rbuflen;
 
     memcpy( &did, ibuf, sizeof( did ));
     ibuf += sizeof( did );
-    if (( dir = dirsearch( vol, did )) == NULL ) {
+    if (( dir = dirlookup( vol, did )) == NULL ) {
         return( AFPERR_NOOBJ );
     }
 
@@ -798,12 +829,14 @@ int		ibuflen, *rbuflen;
         return( AFPERR_NOOBJ );
     }
 
-    if ((*path == '\0') || !(of = of_findname(vol, curdir, path))) {
+
+    upath = mtoupath( vol, path );
+    if ((*path == '\0') || !(of = of_findname(upath, NULL))) {
         memset(&ad, 0, sizeof(ad));
         adp = &ad;
     } else
         adp = of->of_ad;
-    if ( ad_open( mtoupath( vol, path ),
+    if ( ad_open( upath,
                   (( *path == '\0' ) ? ADFLAGS_HF|ADFLAGS_DIR : ADFLAGS_HF),
                   O_RDONLY, 0666, adp) < 0 ) {
         return( AFPERR_NOITEM );
@@ -835,7 +868,7 @@ int		ibuflen, *rbuflen;
     struct vol		*vol;
     struct dir		*dir;
     struct ofork        *of;
-    char		*path;
+    char		*path, *upath;
     u_int32_t		did;
     u_int16_t		vid;
 
@@ -850,7 +883,7 @@ int		ibuflen, *rbuflen;
 
     memcpy( &did, ibuf, sizeof( did ));
     ibuf += sizeof( did );
-    if (( dir = dirsearch( vol, did )) == NULL ) {
+    if (( dir = dirlookup( vol, did )) == NULL ) {
         return( AFPERR_NOOBJ );
     }
 
@@ -858,12 +891,14 @@ int		ibuflen, *rbuflen;
         return( AFPERR_NOOBJ );
     }
 
-    if ((*path == '\0') || !(of = of_findname(vol, curdir, path))) {
+    upath = mtoupath( vol, path );
+    if ((*path == '\0') || !(of = of_findname(upath, NULL))) {
         memset(&ad, 0, sizeof(ad));
         adp = &ad;
     } else
         adp = of->of_ad;
-    if ( ad_open( mtoupath( vol, path ),
+
+    if ( ad_open( upath,
                   (( *path == '\0' ) ? ADFLAGS_HF|ADFLAGS_DIR : ADFLAGS_HF),
                   O_RDWR, 0, adp) < 0 ) {
         switch ( errno ) {
