@@ -1,5 +1,5 @@
 /*
- * $Id: file.h,v 1.3.2.1 2001/12/03 05:01:04 jmarcus Exp $
+ * $Id: file.h,v 1.11 2002/10/09 18:46:31 didg Exp $
  *
  * Copyright (c) 1990,1991 Regents of The University of Michigan.
  * All Rights Reserved.
@@ -38,18 +38,20 @@
 
 extern const u_char	ufinderi[];
 
-#define FILPBIT_ATTR	0
-#define FILPBIT_PDID	1
-#define FILPBIT_CDATE	2
-#define FILPBIT_MDATE	3
-#define FILPBIT_BDATE	4
-#define FILPBIT_FINFO	5
-#define FILPBIT_LNAME	6
-#define FILPBIT_SNAME	7
-#define FILPBIT_FNUM	8
-#define FILPBIT_DFLEN	9
-#define FILPBIT_RFLEN	10
-#define FILPBIT_PDINFO  13    /* ProDOS Info */
+#define FILPBIT_ATTR	 0
+#define FILPBIT_PDID	 1
+#define FILPBIT_CDATE	 2
+#define FILPBIT_MDATE	 3
+#define FILPBIT_BDATE	 4
+#define FILPBIT_FINFO	 5
+#define FILPBIT_LNAME	 6
+#define FILPBIT_SNAME	 7
+#define FILPBIT_FNUM	 8
+#define FILPBIT_DFLEN	 9
+#define FILPBIT_RFLEN	 10
+#define FILPBIT_EXTDFLEN 11
+#define FILPBIT_PDINFO   13    /* ProDOS Info */
+#define FILPBIT_EXTRFLEN 14
 
 /* attribute bits. (d) = directory attribute bit as well. */
 #define ATTRBIT_INVISIBLE (1<<0)  /* invisible (d) */
@@ -57,6 +59,7 @@ extern const u_char	ufinderi[];
 #define ATTRBIT_SYSTEM    (1<<2)  /* system (d) */
 #define ATTRBIT_DOPEN     (1<<3)  /* data fork already open */
 #define ATTRBIT_ROPEN     (1<<4)  /* resource fork already open */
+#define ATTRBIT_SHARED    (1<<4)  /* shared area (d) */
 #define ATTRBIT_NOWRITE   (1<<5)  /* write inhibit(v2)/read-only(v1) bit */
 #define ATTRBIT_BACKUP    (1<<6)  /* backup needed (d) */
 #define ATTRBIT_NORENAME  (1<<7)  /* rename inhibit (d) */
@@ -65,22 +68,21 @@ extern const u_char	ufinderi[];
 #define ATTRBIT_SETCLR	  (1<<15) /* set/clear bits (d) */
 
 struct extmap {
-    struct extmap	*em_next;
-    char		em_ext[ MAXPATHLEN + 1];
+    char		*em_ext;
     char		em_creator[ 4 ];
     char		em_type[ 4 ];
 };
 
-extern struct extmap	*extmap;
 extern struct extmap	*getextmap __P((const char *));
+extern struct extmap	*getdefextmap __P((void));
 
 extern int getfilparams __P((struct vol *, u_int16_t, char *,
                                  struct dir *, struct stat *, char *buf,
                                  int *));
 extern int setfilparams __P((struct vol *, char *, u_int16_t, char *));
-extern int renamefile   __P((char *, char *, char *, const int));
+extern int renamefile   __P((char *, char *, char *, const int, struct adouble *));
 extern int copyfile     __P((char *, char *, char *, const int));
-extern int deletefile   __P((char *));
+extern int deletefile   __P((char *, int));
 
 /* FP functions */
 extern int      afp_exchangefiles __P((AFPObj *, char *, int, char *, int *));
