@@ -1,5 +1,5 @@
 /*
- * $Id: ofork.c,v 1.5.2.2 2002/02/07 23:57:27 srittau Exp $
+ * $Id: ofork.c,v 1.19.2.1 2003/06/09 14:53:16 srittau Exp $
  *
  * Copyright (c) 1996 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
@@ -14,7 +14,7 @@
 #include <string.h>
 #include <sys/stat.h> /* works around a bug */
 #include <sys/param.h>
-#include <syslog.h>
+#include <atalk/logger.h>
 #include <errno.h>
 
 #include <atalk/adouble.h>
@@ -24,7 +24,7 @@
 #include "directory.h"
 #include "fork.h"
 
-/* we need to have a hashed list of oforks (by name). just hash
+/* we need to have a hashed list of oforks (by dev inode). just hash
  * by first letter. */
 #define OFORK_HASHSIZE  64
 static struct ofork     *ofork_table[OFORK_HASHSIZE];
@@ -35,21 +35,22 @@ static u_short		lastrefnum = 0;
 
 
 /* OR some of each character for the hash*/
-static __inline__ unsigned long hashfn(const char *name)
+static __inline__ unsigned long hashfn(const struct file_key *key)
 {
+#if 0
     unsigned long i = 0;
-
     while (*name) {
         i = ((i << 4) | (8*sizeof(i) - 4)) ^ *name++;
     }
-    return i & (OFORK_HASHSIZE - 1);
+#endif    
+    return key->inode & (OFORK_HASHSIZE - 1);
 }
 
 static __inline__ void of_hash(struct ofork *of)
 {
     struct ofork **table;
 
-    table = &ofork_table[hashfn(of->of_name)];
+    table = &ofork_table[hashfn(&of->key)];
     if ((of->next = *table) != NULL)
         (*table)->prevp = &of->next;
     *table = of;
@@ -68,7 +69,7 @@ static __inline__ void of_unhash(struct ofork *of)
 void of_pforkdesc( f )
 FILE	*f;
 {
-    u_short	ofrefnum;
+    int	ofrefnum;
 
     if (!oforks)
         return;
@@ -82,7 +83,7 @@ FILE	*f;
 
 int of_flush(const struct vol *vol)
 {
-    u_int16_t	refnum;
+    int	refnum;
 
     if (!oforks)
         return 0;
@@ -90,55 +91,65 @@ int of_flush(const struct vol *vol)
     for ( refnum = 0; refnum < nforks; refnum++ ) {
         if (oforks[ refnum ] != NULL && (oforks[refnum]->of_vol == vol) &&
                 flushfork( oforks[ refnum ] ) < 0 ) {
-            syslog( LOG_ERR, "of_flush: %s", strerror(errno) );
+            LOG(log_error, logtype_afpd, "of_flush: %s", strerror(errno) );
         }
     }
     return( 0 );
 }
 
-
-int of_rename(vol, olddir, oldpath, newdir, newpath)
+int of_rename(vol, s_of, olddir, oldpath, newdir, newpath)
 const struct vol *vol;
+struct ofork *s_of;
 struct dir *olddir, *newdir;
 const char *oldpath, *newpath;
 {
     struct ofork *of, *next, *d_ofork;
 
-    next = ofork_table[hashfn(oldpath)];
+    if (!s_of)
+        return AFP_OK;
+        
+    next = ofork_table[hashfn(&s_of->key)];
     while ((of = next)) {
         next = next->next; /* so we can unhash and still be all right. */
 
-        if ((vol == of->of_vol) && (olddir == of->of_dir) &&
-                (strcmp(of->of_name, oldpath) == 0)) {
-            of_unhash(of);
+        if (vol == of->of_vol && olddir == of->of_dir &&
+	         s_of->key.dev == of->key.dev && 
+	         s_of->key.inode == of->key.inode ) {
             strncpy( of->of_name, newpath, of->of_namelen);
-            of->of_d_prev->of_d_next = of->of_d_prev;
-            of->of_d_next->of_d_prev = of->of_d_next;
-            of->of_dir = newdir;
-            if (!(d_ofork = newdir->d_ofork)) {
-                newdir->d_ofork = of;
-                of->of_d_next = of->of_d_prev = of;
-            } else {
-                of->of_d_next = d_ofork;
-                of->of_d_prev = d_ofork->of_d_prev;
-                of->of_d_prev->of_d_next = of;
-                d_ofork->of_d_prev = of;
+            if (newdir != olddir) {
+                of->of_d_prev->of_d_next = of->of_d_next;
+                of->of_d_next->of_d_prev = of->of_d_prev;
+                if (of->of_dir->d_ofork == of) {
+                    of->of_dir->d_ofork = (of == of->of_d_next) ? NULL : of->of_d_next;
+                }	    
+                of->of_dir = newdir;
+                if (!(d_ofork = newdir->d_ofork)) {
+                    newdir->d_ofork = of;
+                    of->of_d_next = of->of_d_prev = of;
+                } else {
+                    of->of_d_next = d_ofork;
+                    of->of_d_prev = d_ofork->of_d_prev;
+                    of->of_d_prev->of_d_next = of;
+                    d_ofork->of_d_prev = of;
+                }
             }
-            of_hash(of);
         }
     }
 
     return AFP_OK;
 }
 
+#define min(a,b)	((a)<(b)?(a):(b))
+
 struct ofork *
-            of_alloc(vol, dir, path, ofrefnum, eid, ad)
-            struct vol          *vol;
-struct dir		*dir;
+            of_alloc(vol, dir, path, ofrefnum, eid, ad, st)
+struct vol      *vol;
+struct dir	*dir;
 char		*path;
-u_int16_t		*ofrefnum;
-const int           eid;
-struct adouble      *ad;
+u_int16_t	*ofrefnum;
+const int       eid;
+struct adouble  *ad;
+struct stat     *st;
 {
     struct ofork        *of, *d_ofork;
     u_int16_t		refnum, of_refnum;
@@ -147,6 +158,8 @@ struct adouble      *ad;
 
     if (!oforks) {
         nforks = (getdtablesize() - 10) / 2;
+	/* protect against insane ulimit -n */
+        nforks = min(nforks, 0xffff);
         oforks = (struct ofork **) calloc(nforks, sizeof(struct ofork *));
         if (!oforks)
             return NULL;
@@ -180,14 +193,14 @@ struct adouble      *ad;
     */
     lastrefnum = refnum;
     if ( i == nforks ) {
-        syslog(LOG_ERR, "of_alloc: maximum number of forks exceeded.");
+        LOG(log_error, logtype_afpd, "of_alloc: maximum number of forks exceeded.");
         return( NULL );
     }
 
     of_refnum = refnum % nforks;
     if (( oforks[ of_refnum ] =
                 (struct ofork *)malloc( sizeof( struct ofork ))) == NULL ) {
-        syslog( LOG_ERR, "of_alloc: malloc: %s", strerror(errno) );
+        LOG(log_error, logtype_afpd, "of_alloc: malloc: %s", strerror(errno) );
         return NULL;
     }
     of = oforks[of_refnum];
@@ -196,7 +209,7 @@ struct adouble      *ad;
     if (!ad) {
         ad = malloc( sizeof( struct adouble ) );
         if (!ad) {
-            syslog( LOG_ERR, "of_alloc: malloc: %s", strerror(errno) );
+            LOG(log_error, logtype_afpd, "of_alloc: malloc: %s", strerror(errno) );
             return NULL;
         }
 
@@ -227,25 +240,25 @@ struct adouble      *ad;
     /* here's the deal: we allocate enough for the standard mac file length.
      * in the future, we'll reallocate in fairly large jumps in case
      * of long unicode names */
-    if (( of->of_name =(char *)malloc(MACFILELEN + 1)) ==
-            NULL ) {
-        syslog( LOG_ERR, "of_alloc: malloc: %s", strerror(errno) );
+    if (( of->of_name =(char *)malloc(255 + 1)) == NULL ) {
+        LOG(log_error, logtype_afpd, "of_alloc: malloc: %s", strerror(errno) );
         if (!ad)
             free(of->of_ad);
         free(of);
         oforks[ of_refnum ] = NULL;
         return NULL;
     }
-    strncpy( of->of_name, path, of->of_namelen = MACFILELEN + 1);
+    strncpy( of->of_name, path, of->of_namelen = 255 + 1);
     *ofrefnum = refnum;
-    of->of_refnum = of_refnum;
-    of_hash(of);
-
+    of->of_refnum = refnum;
+    of->key.dev = st->st_dev;
+    of->key.inode = st->st_ino;
     if (eid == ADEID_DFORK)
         of->of_flags = AFPFORK_DATA;
     else
         of->of_flags = AFPFORK_RSRC;
 
+    of_hash(of);
     return( of );
 }
 
@@ -257,21 +270,32 @@ struct ofork *of_find(const u_int16_t ofrefnum )
     return( oforks[ ofrefnum % nforks ] );
 }
 
+/* --------------------------
+*/
 struct ofork *
-            of_findname(const struct vol *vol, const struct dir *dir, const char *name)
+            of_findname(const char *name, struct stat *st)
 {
     struct ofork *of;
+    struct file_key key;
+    struct stat buffer;
+    
+    if (st == NULL) {
+        st = &buffer;
+        if (stat(name, st) < 0)
+            return NULL;
+    }
+    key.dev = st->st_dev;
+    key.inode = st->st_ino;
 
-    for (of = ofork_table[hashfn(name)]; of; of = of->next) {
-        if ((vol == of->of_vol) && (dir == of->of_dir) &&
-                (strcmp(of->of_name, name) == 0))
+    for (of = ofork_table[hashfn(&key)]; of; of = of->next) {
+        if (key.dev == of->key.dev && key.inode == of->key.inode ) {
             return of;
+        }
     }
 
     return NULL;
 }
 
-
 void of_dealloc( of )
 struct ofork	*of;
 {
@@ -287,7 +311,7 @@ struct ofork	*of;
         of->of_dir->d_ofork = (of == of->of_d_next) ? NULL : of->of_d_next;
     }
 
-    oforks[ of->of_refnum ] = NULL;
+    oforks[ of->of_refnum % nforks ] = NULL;
     free( of->of_name );
 
     /* decrease refcount */
