@@ -1,5 +1,5 @@
 /*
- * $Id: server_child.c,v 1.4 2001/09/06 19:04:40 rufustfirefly Exp $
+ * $Id: server_child.c,v 1.7.2.2 2003/11/14 14:25:41 didg Exp $
  *
  * Copyright (c) 1997 Adrian Sun (asun@zoology.washington.edu)
  * All rights reserved. See COPYRIGHT.
@@ -24,19 +24,27 @@
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 #include <signal.h>
-#include <syslog.h>
+#include <atalk/logger.h>
 
 /* POSIX.1 sys/wait.h check */
 #include <sys/types.h>
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif /* HAVE_SYS_WAIT_H */
+#include <sys/time.h>
+
 #ifndef WEXITSTATUS
 #define WEXITSTATUS(stat_val) ((unsigned)(stat_val) >> 8)
 #endif /* ! WEXITSTATUS */
 #ifndef WIFEXITED
 #define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
 #endif /* ! WIFEXITED */
+#ifndef WIFSTOPPED
+#define WIFSTOPPED(status) (((status) & 0xff) == 0x7f)
+#endif
+#ifndef WIFSIGNALED
+#define WIFSIGNALED(status) (!WIFSTOPPED(status) && !WIFEXITED(status)) 
+#endif
 
 #include <atalk/server_child.h>
 
@@ -121,18 +129,18 @@ int server_child_add(server_child *children, const int forkid,
 {
   server_child_fork *fork;
   struct server_child_data *child;
-  sigset_t sig;
+  sigset_t sig, oldsig;
 
   /* we need to prevent deletions from occuring before we get a 
    * chance to add the child in. */
   sigemptyset(&sig);
   sigaddset(&sig, SIGCHLD);
-  sigprocmask(SIG_BLOCK, &sig, NULL);
+  sigprocmask(SIG_BLOCK, &sig, &oldsig);
 
   /* it's possible that the child could have already died before the
    * sigprocmask. we need to check for this. */
   if (kill(pid, 0) < 0) {
-    sigprocmask(SIG_UNBLOCK, &sig, NULL);
+    sigprocmask(SIG_SETMASK, &oldsig, NULL);
     return 1;
   }
 
@@ -140,20 +148,20 @@ int server_child_add(server_child *children, const int forkid,
 
   /* if we already have an entry. just return. */
   if (resolve_child(fork->table, pid)) {
-    sigprocmask(SIG_UNBLOCK, &sig, NULL);
+    sigprocmask(SIG_SETMASK, &oldsig, NULL);
     return 0;
   }
 
   if ((child = (struct server_child_data *) 
        calloc(1, sizeof(struct server_child_data))) == NULL) {
-    sigprocmask(SIG_UNBLOCK, &sig, NULL);
+    sigprocmask(SIG_SETMASK, &oldsig, NULL);
     return -1;
   }
 
   child->pid = pid;
   hash_child(fork->table, child);
   children->count++;
-  sigprocmask(SIG_UNBLOCK, &sig, NULL);
+  sigprocmask(SIG_SETMASK, &oldsig, NULL);
 
   return 0;
 }
@@ -249,19 +257,49 @@ void server_child_handler(server_child *children)
 	break;
       }
     }
-    
     if (WIFEXITED(status)) {
       if (WEXITSTATUS(status)) {
-	syslog(LOG_INFO, "server_child[%d] %d exited %d", i, pid, 
+	LOG(log_info, logtype_default, "server_child[%d] %d exited %d", i, pid, 
 	       WEXITSTATUS(status));
       } else {
-	syslog(LOG_INFO, "server_child[%d] %d done", i, pid);
+	LOG(log_info, logtype_default, "server_child[%d] %d done", i, pid);
       }
     } else {
-      if (WIFSIGNALED(status)) 
-	syslog(LOG_INFO, "server_child[%d] %d killed", i, pid);
+      if (WIFSIGNALED(status))
+      { 
+	LOG(log_info, logtype_default, "server_child[%d] %d killed", i, pid);
+      }
       else
-	syslog(LOG_INFO, "server_child[%d] %d died", i, pid);
+      {
+	LOG(log_info, logtype_default, "server_child[%d] %d died", i, pid);
+      }
     }
   }
 }
+
+/* --------------------------- 
+ * reset children signals
+*/
+void server_reset_signal(void)
+{
+    struct sigaction    sv;
+    sigset_t            sigs;
+    const struct itimerval none = {{0, 0}, {0, 0}};
+
+    setitimer(ITIMER_REAL, &none, NULL);
+    memset(&sv, 0, sizeof(sv));
+    sv.sa_handler =  SIG_DFL;
+    sigemptyset( &sv.sa_mask );
+    
+    sigaction(SIGALRM, &sv, 0 );
+    sigaction(SIGHUP,  &sv, 0 );
+    sigaction(SIGTERM, &sv, 0 );
+    sigaction(SIGCHLD, &sv, 0 );
+    
+    sigemptyset(&sigs);
+    sigaddset(&sigs, SIGALRM);
+    sigaddset(&sigs, SIGHUP);
+    sigaddset(&sigs, SIGCHLD);
+    sigprocmask(SIG_UNBLOCK, &sigs, NULL);
+        
+}
